import requests
import urlparse

from django.contrib.auth import authenticate, login


class BaseOAuth2Client(object):
    """
    Base class that defines the client for OAuth2 services.

    """
    # The name of the service.
    service_name = None

    # The app's client id provided by the service.
    client_id = None

    # The app's secret provided by the service.
    client_secret = None

    # The app's access token provided by the service.
    app_access_token = None

    # The endpoint of the service's api where the auth/login
    # dialog is requested.
    login_dialog_endpoint = None

    # The endpoint of the service's api where the access token is requested.
    access_token_endpoint = None

    # The permissions that the app will ask for.
    scope = None

    # Is the email used as username in the site?
    email_is_username = None

    # The model where the profiles are stored.
    model = None

    # The key for the session dictionary where the flow data will be stored.
    session_key = None

    def login(self, request, uid):
        """
        Logs the user in.

        """
        login(request, authenticate(**{'service_uid': uid}))

    def get_callback_url(self):
        """
        Returns the absolute url that will be passed as callback argument
        for the OAuth login/auth dialog.

        Subclasses must implement this method.

        """
        raise NotImplementedError

    def request_access_token(self, code, method='GET'):
        """
        Requests the access token to the service provider. If the access
        token is valid, stores it in the current user's session.

        """
        params = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'redirect_uri': self.get_callback_url(),
            'code': code
        }

        if method.upper() == 'GET':
            response = requests.get(
                self.access_token_endpoint, params=params
            )

        elif method.upper() == 'POST':
            response = requests.post(
                self.access_token_endpoint, params=params
            )

        return self.parse_access_token(
            response.content
        ).get('access_token', None)

    def parse_access_token(self, content):
        """
        Parses the query string returned by the service as the response of
        the access token request.

        """
        return dict(urlparse.parse_qsl(content))

    def debug_access_token(self):
        """
        Checks if the given access token is valid.

        Should update the access_token_expires_at, and uid atrributes.
        Should return a boolean.

        """
        raise NotImplementedError

    def request(
        self, endpoint, method='GET', params={}, headers={}, signature=None):
        """
        Makes a request against an api endpoint. If the request needs to be
        signed, the signature is required to be a dictionary.

        """
        if signature:
            params.update(signature)

        if method.upper() == 'GET':
            response = requests.get(
                endpoint,
                params=params
            )

        elif method.upper() == 'POST':
            response = requests.post(
                endpoint,
                params=params, headers=headers
            )

        return response.json()



from datetime import datetime

from django.contrib import messages
from django.contrib.auth.models import User
from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import View

from socialnetwork.signals import connect, disconnect, login


class BaseOAuthView(View):
    """
    Base class for the OAuth flow.
    OAuth views must be inherited from this class.

    """
    # The client that defines the necessary information to perform
    # the OAuth flow.
    client = None

    def __get_session(self):
        """
        Returns the current session's object to store the data that will
        be used trough the OAuth flow.

        """
        if self.client.session_key not in self.__request.session:
            self.__request.session[self.client.session_key] = {}

        return self.__request.session[self.client.session_key]

    def __set_session(self, dictionary):
        """
        Updates the current session's object, useful if the backend is not
        configured to save session in every request.

        """
        self.__request.session[self.client.session_key] = dictionary

    def session_get(self, key):
        """
        Returns the provided key value from the current session's object. If
        key is not in the object returns None.

        """
        return self.__get_session().get(key, None)

    def session_put(self, **dictionary):
        """
        Puts the given dictionary into the current session's object.

        """
        session = self.__get_session()
        session.update(**dictionary)
        self.__set_session(session)

    def session_pop(self, key):
        """
        Returns the provided key value from the current session's object and
        removes the key from the object. If key is not in the object returns
        None.

        """
        session = self.__get_session()
        value = session.pop(key, None)
        self.__set_session(session)
        return value

    def session_clear(self):
        """
        Removes the object where the flow information of the service are
        stored from the current user's session.

        """
        if self.client.session_key in self.__request.session:
            del self.__request.session[self.client.session_key]

    def get_redirect_url(self):
        """"
        Returns the url where the user will be redirected after its request
        is processed.

        Subclasses must implement this method.

        """
        raise NotImplementedError

    def dispatch(self, request, *args, **kwargs):
        """
        Sets the current request as an atrribute, this way it can be accessed
        easily by the methods that modifies the user session.

        """
        self.__request = request
        return super(BaseOAuthView, self).dispatch(request, *args, **kwargs)


class OAuthDialogRedirectView(BaseOAuthView):
    """
    Base view that handles the service's auth/login dialog request.

    """
    def get(self, request):
        # Clears the current user's session to avoid the use of old
        # login information.
        self.session_clear()

        # Fetches the url where the user will be redirected when the
        # flow ends and stores it in the user's session.
        if 'next' in request.GET:
            self.session_put(**{
                'next': request.GET['next']
            })

        else:
            self.session_put(**{
                'next': '/'
            })

        # Makes the request of the login/auth dialog to the service.
        return redirect(self.get_redirect_url())


class OAuthCallbackView(BaseOAuthView):
    """
    Base view that handles the callback redirection from the service.

    """
    def get(self, request):
        # Protect the view to be accessed by non OAuth requests.
        if 'code' not in request.GET:
            return HttpResponseForbidden()

        # Request the access token to the service
        token = self.client.request_access_token(request.GET['code'])

        # If the given access token is valid then tries to login or create
        # the user, if the token is invalid then redirects to the start of
        # the flow.
        token_is_valid, token_debug = self.client.debug_access_token(token)

        if token_is_valid:
            self.session_put(**{
                'access_token': token,
                'expires_at': token_debug['expires_at'],
                'uid': token_debug['user_id'],
            })

            # Gets or creates the profile in the service
            profile, created = self.client.model.objects.get_or_create(
                service_uid=self.session_get('uid')
            )

            # If the given access_token does not match the token stored in
            # the db updates the profile with the new access token.
            if profile.oauth_access_token != token:
                profile.oauth_access_token = token
                profile.oauth_access_token_expires_at = (
                    datetime.fromtimestamp(self.session_get('expires_at'))
                )
                profile.save()

            # If the profile was created or has no user, tries to attach the
            # the profile to current user, if the user is not logged in
            # redirects it to the final setup view to create a new one.
            if created or not profile.user:
                if request.user.is_authenticated():
                    profile.user = self.request.user
                    profile.save()
                    connect.send(
                        sender=self, user=self.request.user,
                        service=self.client.service_name.lower()
                    )
                    messages.success(self.request, _('Your %(service)s '
                        'profile was successfully connected with your '
                        'user account.' % {
                            'service': self.client.service_name
                        })
                    )
                else:
                    self.session_put(**{'new_user': True})

            elif (profile.user and profile.user != self.request.user and
                  self.request.user.is_authenticated()):
                    messages.error(self.request, _('This %(service)s profile '
                        'is already connected with another user account.' % {
                            'service': self.client.service_name
                        })
                    )
                    return redirect(self.get_redirect_url())

            else:
                # Logs the user in if it is not logged yet.
                self.client.login(request, self.session_get('uid'))
                login.send(
                    sender=self, user=self.request.user,
                    service=self.client.service_name.lower()
                )

            return redirect(self.get_redirect_url())


class OAuthSetupView(BaseOAuthView):
    """
    Base View that handles the setup for the account after the access
    token is provided.

    """
    def retrieve_user_data(self):
        """
        Returns a dictionary with the keys 'first_name', 'last_name', 'email'
        and 'username'. The values for the keys should be retrieved from the
        service.

        Subclasses must implement this method.

        """
        raise NotImplementedError

    def get(self, request):
        # Protect the view to be accessed by non logged users.
        if request.user.is_authenticated():
            return HttpResponseForbidden()

        # Fetches the profile that corresponds to the service uid.
        profile = self.client.model.objects.get(service_uid=self.session_get('uid'))

        # If the profile has no user tries to fetch an already registered
        # user with the retrieved username/email. If the looked user does
        # not exist creates a new one with all the retrieved data.
        if not profile.user:
            data = self.retrieve_user_data()

            try:
                user = User.objects.get(
                    username=data['username'], email=data['email']
                )
            except User.DoesNotExist:
                user = User.objects.create(**data)

            profile.user = user
            profile.save()
            connect.send(
                sender=self, user=user,
                service=self.client.service_name.lower()
            )

        # Authenticates the new user.
        self.client.login(request, self.session_get('uid'))
        login.send(
            sender=self, user=user,
            service=self.client.service_name.lower()
        )

        # Redirects the user to the proper url.
        return redirect(self.get_redirect_url())


class BaseProfileDisconnectView(View):
    """
    Base view that handles social profiles disconnection.

    """
    client = None

    def get_profile(self):
        """
        Returns the current profile object that will be deleted.
        Subclasses must implement this method.

        """
        raise NotImplementedError

    def post(self, request, *args, **kwargs):
        profile = self.get_profile()
        if profile:
            print profile.__dict__
            user = self.request.user
            profile.delete()
            disconnect.send(
                sender=self, user=user,
                service=self.client.service_name.lower()
            )

        messages.success(self.request, _('Your %(service)s profile was '
            'successfully disconnected from your user account.' % {
                'service': self.client.service_name
            })
        )

        if 'next' in request.POST:
            return redirect(request.POST['next'])
        else:
            return redirect('/')



from django import template
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

register = template.Library()


@register.simple_tag(takes_context=True)
def facebook_login(context, text=None, css_class=None):

    context['value'] = text or _('Sign in with Facebook')
    context['action'] = reverse('socialnetwork:facebook:login')
    context['css_class'] = css_class

    return template.loader.render_to_string(
        'login/login_button.html', context
    )



from django.conf.urls.defaults import url, patterns

from socialnetwork.facebook import views


urlpatterns = patterns('',
    url(
        r'^login/$',
        views.FacebookDialogRedirect.as_view(), name='login'
    ),
    url(
        r'^callback/$',
        views.FacebookCallback.as_view(), name='callback'
    ),
    url(
        r'^setup/$',
        views.FacebookSetup.as_view(), name='setup'
    ),
    url(
        r'^disconnect/$',
        views.FacebookProfileDisconnect.as_view(), name='disconnect'
    ),
)



import urllib

from django.core.urlresolvers import reverse

from socialnetwork.base.views import (
    OAuthDialogRedirectView, OAuthCallbackView,
    OAuthSetupView, BaseProfileDisconnectView
)
from socialnetwork.facebook.clients import FacebookClient, FacebookGraph


class FacebookDialogRedirect(OAuthDialogRedirectView):
    client = FacebookClient()

    def get_redirect_url(self):
        return '%(url)s%(params)s' % {
            'url': self.client.login_dialog_endpoint,
            'params': urllib.urlencode({
                'client_id': self.client.client_id,
                'redirect_uri': self.client.get_callback_url(),
                'scope': self.client.scope,
            })
        }


class FacebookCallback(OAuthCallbackView):
    client = FacebookClient()

    def get_redirect_url(self):
        if self.session_get('new_user'):
            return reverse('socialnetwork:facebook:setup')

        else:
            redirect_url = self.session_get('next')
            self.session_clear()
            return redirect_url or '/'


class FacebookSetup(OAuthSetupView):
    client = FacebookClient()

    def get_redirect_url(self):
        redirect_url = self.session_get('next')
        self.session_clear()
        return redirect_url or '/'

    def retrieve_user_data(self):
        graph = FacebookGraph(
            self.client.model.objects.get(
                service_uid=self.session_get('uid')
            )
        )

        fields = ['first_name', 'last_name', 'email', 'username']

        data = graph.get('me', params={'fields': ','.join(fields)})

        if self.client.email_is_username:
            data['username'] = data['email']

        return dict(zip(fields, [data[f] for f in fields]))


class FacebookProfileDisconnect(BaseProfileDisconnectView):
    client = FacebookClient()

    def get_profile(self):
        try:
            return self.request.user.facebookprofile
        except:
            return None



from django.utils.translation import ugettext_lazy as _

from socialnetwork.base.models import BaseOAuth2Profile


class FacebookProfile(BaseOAuth2Profile):
    class Meta:
        verbose_name = _('facebook profile')
        verbose_name_plural = _('facebook profiles')



from django.conf import settings


CLIENT_ID = getattr(
    settings, 'SOCIALNETWORK_FACEBOOK_CLIENT_ID', None
)

CLIENT_SECRET = getattr(
    settings, 'SOCIALNETWORK_FACEBOOK_CLIENT_SECRET', None
)

APP_ACCESS_TOKEN = getattr(
    settings, 'SOCIALNETWORK_FACEBOOK_APP_ACCESS_TOKEN', None
)

EMAIL_IS_USERNAME = getattr(
    settings, 'SOCIALNETWORK_FACEBOOK_EMAIL_IS_USERNAME', True
)

SCOPE = getattr(
    settings, 'SOCIALNETWORK_FACEBOOK_SCOPE_PERMISSIONS', ['email']
)



from django.contrib.auth.backends import ModelBackend


class BaseSocialBackend(ModelBackend):
    """
    Base backend that handles the login with the social networks.

    """
    supports_object_permissions = False
    supports_anonymous_user = False

    # The model to lookup for the user
    model = None

    def authenticate(self, service_uid):
        try:
            profile = self.model.objects.get(
                service_uid=service_uid
            )
            return profile.user

        except self.model.DoesNotExist:
            return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

from socialnetwork import __version__


setup(
    name='django-socialnetwork',
    version=__version__,
    description=('Provides functionality to login and share with social '
        'networks to Django.'),
    long_description=open(
        os.path.join(os.path.dirname(__file__), 'README.rst')
    ).read(),
    author=u'Gabriel González',
    author_email='zurg.cei@gmail.com',
    url='https://github.com/gGonz/django-socialnetwork',
    license='Apache License (2.0)',
    install_requires=[
        'Django >= 1.4',
        'requests',
        'requests_oauthlib',
    ],
    packages=find_packages(),
    include_package_data=True,
    classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
	'License :: OSI Approved :: Apache Software License',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
	'Topic :: Software Development :: Libraries :: Python Modules',
	'Topic :: Internet',

    ]
)




import requests

from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

from socialnetwork.base.clients import BaseOAuth2Client
from socialnetwork.facebook import settings
from socialnetwork.facebook.models import FacebookProfile


class FacebookClient(BaseOAuth2Client):
    service_name = 'Facebook'

    # Facebook configuration loaded from the settings.
    client_id = settings.CLIENT_ID
    client_secret = settings.CLIENT_SECRET
    app_access_token = settings.APP_ACCESS_TOKEN
    scope = ','.join(settings.SCOPE)
    email_is_username = settings.EMAIL_IS_USERNAME

    # Facebook manual configuration.
    login_dialog_endpoint = 'https://www.facebook.com/dialog/oauth?'
    access_token_endpoint = 'https://graph.facebook.com/oauth/access_token?'
    api_entrypoint = 'https://graph.facebook.com/'
    session_key = 'socialnetwork:facebook'
    model = FacebookProfile

    # Facebook callback url.
    def get_callback_url(self):
        return 'http://%(domain)s%(callback)s' % {
            'domain': Site.objects.get_current(),
            'callback': reverse('socialnetwork:facebook:callback')
        }

    # Returns the app access token if it is defined in settings,
    # otherwise fetches the token from Facebook.
    def get_app_access_token(self):
        if self.app_access_token:
            return self.app_access_token

        else:
            response = requests.get(
                'https://graph.facebook.com/oauth/access_token?',
                params={
                    'client_id': self.client_id,
                    'client_secret': self.client_secret,
                    'grant_type': 'client_credentials'
                }
            )

            content = self.parse_access_token(response.content)
            return content['access_token']

    # Connects with Facebook to debug the given access token.
    def debug_access_token(self, token):
        response = self.request(
            self.api_entrypoint + 'debug_token?',
            params={'input_token': token},
            signature={'access_token': self.get_app_access_token()}
        )

        if 'data' in response and response['data']['is_valid']:
            return (True, response['data'])
        else:
            return (False, None)


class FacebookGraph(FacebookClient):
    """
    Client to explore the Facebook Graph API.

    This client should not be used directly, it is defined as an attribute
    in the FacebookClient class.

    """
    def __init__(self, profile):
        self.profile = profile

    def debug_access_token(self):
        return super(FacebookGraph, self).debug_access_token(self.profile.oauth_access_token)

    def compose_signature(self):
        """
        Returns the proper signature for the oauth requests.

        """
        return {'access_token': self.profile.oauth_access_token}

    def get(self, endpoint, params={}, headers={}, signature=None):
        """
        GET method to the Facebook Graph API.

        """
        return self.request(
            self.api_entrypoint + endpoint,
            params=params, headers=headers,
            signature=signature or self.compose_signature()
        )



from django.conf.urls.defaults import url, patterns, include


urlpatterns = patterns('',
    url(
        r'^facebook/',
        include('socialnetwork.facebook.urls', namespace='facebook')
    ),
)
